% DESC: Furjeov red (Fourier Series - FS), Brza Furjeova Transformacija (Fast Fourier 
%   Transform - FFT)
% DEPS: fourier_ser.m
%

% Default settings & package loads
defset;
pkg load signal;
pkg load control;

% Anonymous functions
fcn_sin = @(mag, dc, f, t, ph) mag*sin(2*pi*f.*t + ph*pi/180) + dc;
fcn_square  = @(mag, dc, f, t, ph) mag*square(2*pi*f.*t + ph*pi/180) + dc;
fcn_fseries = @(ao, an, bn, fo, t) ...
  ao + sum(an'.*cos((1:numel(an))'*2*pi*fo.*t) + bn'.*sin((1:numel(bn))'*2*pi*fo.*t));
fcn_triag = @(mag, dc, f, t, ph) mag*sawtooth(2*pi*f.*t + ph*pi/180) + dc;

% Inputs
K = 20;
N = 2^16;
fs = N/2;
fo = 100;
dc = 0;
mag = 1;
ph = 0;
t = (0:N-1)./fs;

% Signal
y = fcn_square(mag, dc, fo, t, ph);

% Time samples
dt = 1/fs;
T = 1/fo;
to_bin = find(t >= T, 1, "first");

% Fourier series
A0 = (1/T)*sum(y(1:to_bin)*dt);
An = zeros(1, K);
Bn = zeros(1, K);
for k=1:K
  An(k) = (2/T)*sum(y(1:to_bin).*cos(2*pi*k*fo*t(1:to_bin))*dt);
  Bn(k) = (2/T)*sum(y(1:to_bin).*sin(2*pi*k*fo*t(1:to_bin))*dt);
endfor

X = [A0, sqrt(An.^2 + Bn.^2)];
k = [0, 1:K];

% Reconstruct signal
yr = fcn_fseries(A0, An, Bn, fo, t);
fser = fourier_ser(y, t, T, K);
fprintf("\n\tA0: %f\t\tX0: %f\n", [A0, X(1)]);
fprintf("\n\tAn:\t\tBn:\t\tXn:\n");
disp( [An' Bn' X(2:end)'] );

% FFT
fr = fs/N; 
f = (0:fr:fs/2); 
fo_bin = round(fo/fr) + 1; 
Y = fft(y, N)/N; 
Y = Y(1) + 2*Y(2:N/2+1); 
f = (0:N/2-1)*fr; 

% Plot
subplot(2,2,1);
stem(f/fo, abs(Y));
xlim([0, K]);
xlabel("{\\it k}");
title("FFT spektar")
subplot(2,2,3);
stem(k, X);
xlabel("Indeks harmonika");
title("Spektar X_n = sqrt(A_n^2 + B_n^2), n > 0, X_0 = A_0", "interpreter", "tex");
xlabel("{\\it k}");
subplot(2,2,[2,4]);
%plot(t, [y', yr', fser.yr']);
plot(t, [y', fser.yr']);
xlim([0,t(to_bin)]);
xlabel("{\\it t} [s]");
title("Vremenski domen");

% Seters
hl = legend("Originalni",  sprintf("Konstruisan (Harmonik %u)", K));
hf = gcf;
fp = get(hf, "position");
pause(1);
set(hl, "box", "off", "location", "northoutside");
set(hf, "position", [fp(1:2), fp(3)*2, fp(4)]);






