% DESC: GUI program za ispitivanje talasnih oblika i spektra karateristicnih tipova
%   signala u elektronici.
% DEPS: defset.m, fourier_ser.m, update_plot.m
%

% Default settings & package loads
defset;
pkg load control;
pkg load signal;
graphics_toolkit qt;
%graphics_toolkit fltk;

% Main figure (window)
fg = figure;

% Structure for holding GUI elements
h = struct;

% Panels
h.panel = uipanel ("title", "Kontrole", "position", [0.025 0.025 0.95 0.35]);
h.panel1 = uipanel ("title", "Grafici", "position", [0.025 0.375 0.95 0.625]);

% Sliders
h.frequency_label = uicontrol ("style", "text",
                           "units", "normalized",
                           "string", "Frekvencija:",
                           "horizontalalignment", "left",
                           "parent", h.panel,
                           "position", [0.5 0.9 0.4 0.1]);

h.frequency_slider = uicontrol ("style", "slider",
                            "units", "normalized",
                            "string", "slider",
                            "callback", @update_plot,
                            "max", 1,
                            "min", 0.05,
                            "value", 0.05,
                            "parent", h.panel,
                            "position", [0.5 0.8 0.4 0.1]);
    
h.mag_label = uicontrol (  "style", "text",
                           "units", "normalized",
                           "string", "Amplituda:",
                           "horizontalalignment", "left",
                           "parent", h.panel,
                           "position", [0.5 0.65 0.4 0.1]);
                           
h.mag_slider = uicontrol (  "style", "slider",
                            "units", "normalized",
                            "string", "slider",
                            "callback", @update_plot,
                            "value", 1,
                            "parent", h.panel,
                            "position", [0.5 0.55 0.4 0.1]);

h.ph_label = uicontrol (   "style", "text",
                           "units", "normalized",
                           "string", "Faza:",
                           "horizontalalignment", "left",
                           "parent", h.panel,
                           "position", [0.5 0.4 0.4 0.1]);
                           
h.ph_slider = uicontrol (  "style", "slider",
                            "units", "normalized",
                            "string", "slider",
                            "min", 0,
                            "max", 360,
                            "value", 0,
                            "callback", @update_plot,
                            "parent", h.panel,
                            "position", [0.5 0.3 0.4 0.1]);

h.offset_label = uicontrol (  "style", "text",
                           "units", "normalized",
                           "string", "Ofset:",
                           "horizontalalignment", "left",
                           "parent", h.panel,
                           "position", [0.5 0.15 0.4 0.1]);
                           
h.offset_slider = uicontrol (  "style", "slider",
                            "units", "normalized",
                            "string", "slider",
                            "callback", @update_plot,
                            "min", 0,
                            "max", 1,
                            "value", 0,
                            "parent", h.panel,
                            "position", [0.5 0.05 0.4 0.1]);

h.harm_label = uicontrol (  "style", "text",
                           "units", "normalized",
                           "string", "Harmonik:",
                           "horizontalalignment", "left",
                           "parent", h.panel,
                           "position", [0.15 0.15 0.25 0.1]);
                           
h.harm_slider = uicontrol (  "style", "slider",
                            "units", "normalized",
                            "string", "slider",
                            "userdata", [0 1 2 3 4 5 6 7 8 9 10],
                            "callback", @update_plot,
                            "min", 1,
                            "max", 40,
                            "value", 1,
                            "parent", h.panel,
                            "position", [0.15 0.05 0.25 0.1]);

% Rectifier control
h.rect_checkbox = uicontrol (  "style", "checkbox",
                             "units", "normalized",
                             "string", "Rect.",
                             "value", 0,
                             "parent", h.panel,
                             "callback", @update_plot,
                             "position", [0.15 0.85 0.1 0.1]);

h.rect_toggle = uicontrol ( "style", "togglebutton",
                            "units", "normalized",
                            "string", "Half-wave",
                            "callback", @update_plot,
                            "parent", h.panel,
                            "position", [0.15 0.7 0.1 0.1]);
                            
% Radiobuttons
h.signal_radio_sin = uicontrol ( "style", "radiobutton",
                                 "units", "normalized",
                                 "string", "sin",
                                 "callback", @update_plot,
                                 "parent", h.panel,
                                 "position", [0.025 0.8 0.1 0.2]);

h.signal_radio_square = uicontrol ("style", "radiobutton",
                                   "units", "normalized",
                                   "string", "square",
                                   "callback", @update_plot,
                                   "value", 0,
                                   "parent", h.panel,
                                   "position", [0.025 0.65 0.1 0.2]);

h.signal_radio_triag = uicontrol ( "style", "radiobutton",
                                   "units", "normalized",
                                   "string", "triag",
                                   "callback", @update_plot,
                                   "value", 0,
                                   "parent", h.panel,
                                   "position", [0.025 0.5 0.1 0.2]);

h.signal_radio_rand = uicontrol ( "style", "radiobutton",
                                   "units", "normalized",
                                   "string", "rand",
                                   "callback", @update_plot,
                                   "value", 0,
                                   "parent", h.panel,
                                   "position", [0.025 0.35 0.1 0.2]);

h.signal_radio_aper = uicontrol (  "style", "radiobutton",
                                   "units", "normalized",
                                   "string", "aper",
                                   "callback", @update_plot,
                                   "value", 0,
                                   "parent", h.panel,
                                   "position", [0.025 0.2 0.1 0.2]);

% Checkboxes                                 
h.db_checkbox = uicontrol (  "style", "checkbox",
                             "units", "normalized",
                             "string", "dB",
                             "value", 0,
                             "parent", h.panel,
                             "callback", @update_plot,
                             "position", [0.025 0.125 0.1 0.1]);


                             
h.log_checkbox = uicontrol ( "style", "checkbox",
                             "units", "normalized",
                             "string", "log(f)",
                             "value", 0,
                             "parent", h.panel,
                             "callback", @update_plot,
                             "position", [0.025 0.025 0.1 0.1]);

h.harm_checkbox = uicontrol ( "style", "checkbox",
                             "units", "normalized",
                             "string", "Furjeov red (aproks.)",
                             "value", 0,
                             "parent", h.panel,
                             "callback", @update_plot,
                             "position", [0.15 0.25 0.25 0.1]);

% Text outputs                           
h.avg_text    = uicontrol ( "style", "text",
                            "units", "normalized",
                            "string", "Srednja vr.:",
                            "horizontalalignment", "left",
                            "parent", h.panel,
                            "callback", @update_plot,
                            "position", [0.3 0.85 0.165 0.1]);
                            
h.eff_text    = uicontrol ( "style", "text",
                            "units", "normalized",
                            "string", "Efektivna vr.:",
                            "horizontalalignment", "left",
                            "parent", h.panel,
                            "callback", @update_plot,
                            "position", [0.3 0.7 0.165 0.1]);

h.fft_text    = uicontrol (  "style", "text",
                            "units", "normalized",
                            "string", "FFT:",
                            "horizontalalignment", "left",
                            "verticalalignment", "top",
                            "parent", h.panel,
                            "callback", @update_plot,
                            "position", [0.3 0.4 0.165 0.25]);

% Axses                         
pause(1);
h.ax =  axes ("parent", h.panel1, "position", [0.05 0.125 0.425 0.8] );
h.ax1 = axes ("parent", h.panel1, "position", [0.55 0.55 0.425 0.375]);
h.ax2 = axes ("parent", h.panel1, "position", [0.55 0.125 0.425 0.375]);


% Anonymus functions
h.fcn_sin = @(mag, dc, f, t, ph) mag*sin(2*pi*f.*t + ph*pi/180) + dc;
h.fcn_square  = @(mag, dc, f, t, ph) mag*square(2*pi*f.*t + ph*pi/180) + dc;
h.fcn_triag = @(mag, dc, f, t, ph) mag*sawtooth(2*pi*f.*t + ph*pi/180) + dc;
h.fcn_rand = @(mag, dc, t) mag*(2*rand(1, numel(t)) - 1) + dc;
%h.fcn_aper = @(mag, dc, f, t, ph) mag*sinc(2*pi*f*t - ph/360) + dc;
h.fcn_aper = @(mag, dc, f, t, ph) mag*rectpuls(t, (ph/360)*(1/f)) + dc;
%h.fcn_aper = @(mag, dc, t, ph) mag*rectpuls(t, 0.0175*ph/360) + dc;
h.fcn_fser = @(y, t, to, k) fourier_ser(y, t, to, k);
h.fcn_average_p = @(y, fo, fs) fo*sum(y(1:round(fs/fo)))./fs; 
h.fcn_effective_p = @(y, fo, fs) sqrt(fo*sum(y(1:round(fs/fo)).^2)./fs);
h.fcn_average_ap = @(y, fo, fs) sum(y(1:round(fs/fo)))./N; 
h.fcn_effective_ap = @(y, fo, fs) sqrt(sum(y(1:round(fs/fo)).^2)./N);

% Seters                   
set(fg, "color", get(0, "defaultuicontrolbackgroundcolor"));
set(fg, "position", [200, 100, 1000, 700]);
set(fg, "Name", "Signali i spektar");

% Update main figure
guidata (fg, h);

% Callback functions calls
update_plot(fg, true);
