function update_plot(obj, init = false)
  % Default values
  h = guidata (obj);
  replot = false;
  recalc = false;

  % gcbo holds the handle of the control
  switch (gcbo)
    case {h.signal_radio_sin}
      set (h.signal_radio_square, "value", 0);
      set (h.signal_radio_triag, "value", 0);
      set (h.signal_radio_rand, "value", 0);
      set (h.signal_radio_aper, "value", 0);
      recalc = true;
    case {h.signal_radio_square}
      set (h.signal_radio_sin, "value", 0);
      set (h.signal_radio_triag, "value", 0);
      set (h.signal_radio_rand, "value", 0);
      set (h.signal_radio_aper, "value", 0);
      recalc = true;
    case {h.signal_radio_triag}
      set (h.signal_radio_sin, "value", 0);
      set (h.signal_radio_square, "value", 0);
      set (h.signal_radio_rand, "value", 0);
      set (h.signal_radio_aper, "value", 0);
      recalc = true;
    case {h.signal_radio_rand}
      set (h.signal_radio_sin, "value", 0);
      set (h.signal_radio_square, "value", 0);
      set (h.signal_radio_triag, "value", 0);
      set (h.signal_radio_aper, "value", 0);
      recalc = true;
    case {h.signal_radio_aper}
      set (h.signal_radio_sin, "value", 0);
      set (h.signal_radio_square, "value", 0);
      set (h.signal_radio_triag, "value", 0);
      set (h.signal_radio_rand, "value", 0);
      recalc = true;
    case {h.frequency_slider}
      recalc = true;
    case {h.db_checkbox}
      cbval = get( gcbo, "value");
      recalc = true;
    case {h.log_checkbox}
      recalc = true; 
    case {h.mag_slider}
%      mag = get(h.mag_slider, "value");
      recalc = true;
    case {h.offset_slider}
%      dc = get(h.offset_slider, "value");
      recalc = true;
    case {h.rect_toggle}
      recalc = true;
    otherwise 
      recalc = true;
  endswitch

  if (recalc || init)
    
    Fo = 1000; % Hz
    N = 2^18;
    fs = N/2; %100e3;
    fres = fs/N;
    t = (0:N-1)./fs;
    ph = get(h.ph_slider, "value");
    mag = get(h.mag_slider, "value");
    dc = get(h.offset_slider, "value");
    fo_norm = get(h.frequency_slider, "value");
    harm = round(get(h.harm_slider, "value"));
    fo_bin = round(fo_norm*Fo/fres);
%    fo = round(fo_bin * fres);
    fo = (fo_bin * fres);
    fax = fres*(0:N/2-1);
    yr = nan(1, numel(t));
    y = nan(1, numel(t));
    Xr = 0;
    thd = 0;
    thd_label = "";
    idx = 0;
    
    set (h.frequency_label, "string", sprintf ("Frequency: %u Hz", fo));
    set (h.mag_label, "string", sprintf ("Magnitude: %3.3f [V]", mag));
    set (h.offset_label, "string", sprintf ("Offset: %3.3f [V]", dc));
    set (h.ph_label, "string", sprintf( "Faza: %3.3f [deg]", ph));
    set (h.harm_label, "string", sprintf( "Harmonik: %u", harm));
    
    if ( get( h.signal_radio_sin, "value" ) )
      y = h.fcn_sin(mag, dc, fo, t, ph);
    elseif (get (h.signal_radio_square, "value" ) )
      y = h.fcn_square(mag, dc, fo, t, ph);
    elseif (get (h.signal_radio_triag, "value" ) )
      y = h.fcn_triag(mag, dc, fo, t, ph);
    elseif (get (h.signal_radio_rand, "value" ) )
      y = h.fcn_rand(mag, dc, t);
    elseif (get (h.signal_radio_aper, "value" ) )
%      y = h.fcn_aper(mag, dc, t, ph);
      y = h.fcn_aper(mag, dc, fo, t, ph);
    else 
      y = h.fcn_sin(mag, dc, fo, t, ph);
    endif
    
    % Rectify

%    disp(get(h.rect_toggle, "selected"));  
    if (get(h.rect_checkbox, "value"))
      if (get(h.rect_toggle, "value"))
        set(h.rect_toggle, "string", "Full-wave");
        y = y - dc;
        idx_p = find(y <= 0);
        idx_m = setdiff((1:N),idx);
        y(idx_p) = -y(idx_p);
        y = y + dc;
        idxf = find(abs(y - dc) < 0.02);
        y(idxf) = dc;
      else
        set(h.rect_toggle, "string", "Half-wave");
        idx = find(y <= dc);
        y(idx) = dc;
      end
    end
    
    % Power calc
    avg = h.fcn_average_p(y, fo, fs);
    eff = h.fcn_effective_p(y, fo, fs);
    
    % FFT
    Y = fft(y, N)./N;
    Yh = [Y(1), 2*Y(2:N/2)];
    YhAbs = abs(Yh);
    YhdB = mag2db(abs(YhAbs));
%    YhAng = angle(Yh)*180/pi;
    YhAng = arg(Yh)*180/pi;
      
    if (init)
      set( h.signal_radio_sin, "value", 1);
      y = h.fcn_sin(mag, dc, fo, t, ph);
      h.plot = plot(t, [y' yr'], "parent", h.ax);
      h.plot1 = plot(fax, YhAbs, "parent", h.ax1);
      h.plot2 = plot(fax, YhAng, "parent", h.ax2);
      ax1_ylabel = "Amplituda";
      Ym = YhAbs;
%      guidata (obj, h);
    else
      if (get(h.db_checkbox, "value"))
        Ym = YhdB;
        ax1_ylabel = "Amplituda [dB]";
      else
        Ym = YhAbs;
        ax1_ylabel = "Amplituda";
      endif
      
      if (get(h.harm_checkbox, "value"))
        fser = h.fcn_fser(y, t, 1/fo, harm);
        yr = fser.yr;
        if ( ~get(h.signal_radio_aper, "value") && ~get(h.signal_radio_rand, "value") ) 
          if (harm > 1)
            Xr = fser.X(2:end);
            thd = 100*sqrt(sum(Xr(2:harm).^2))/Xr(1);
            set (h.harm_label, "string", ...
              sprintf( "Harmonik: %u (THD = %3.2f%%)", harm, thd));
          endif
        endif
      endif
      
      if (get(h.log_checkbox, "value"))
        set(h.ax1, "xscale", "log");
        set(h.ax2, "xscale", "log");
      else
        set(h.ax1, "xscale", "linear");
        set(h.ax2, "xscale", "linear");
      endif
      
      h.ln = findobj(h.ax, "type", "line");
      set(h.ln(2), "ydata", y);
      set(h.ln(1), "ydata", yr);      
      set (h.plot1, "ydata", Ym);
      set (h.plot2, "ydata", YhAng);

    endif
  
    set (h.avg_text, "string", sprintf( "Srednja vr.: %3.3f [deg]", avg));
    set (h.eff_text, "string", sprintf( "Efektivna vr.: %3.3f [deg]", eff));
    set (h.fft_text,  "string", ...
      sprintf( "FFT @f = %3.3f [Hz]:\n- Faza: %3.3f [deg]\n- Amplituda: %3.3f ", ...
      fo, YhAng(fo_bin+1), Ym(fo_bin+1)) );
%    set (h.ax1, "xlim", [0, 20*fo]);
%    guidata (obj, h);   
  endif

  set (h.ax, "xlim", [0, 20/Fo], "xlabel", "{\\it t} [sec]", ...
    "title", "Vremenski domen");
  set (h.ax1, "xlim", [0, 10*Fo], "xticklabel", [], ...
      "title", "Frekvencijski domen (Spektar)", "ylabel", ax1_ylabel);
  set (h.ax2, "xlim", [0, 10*Fo], "xlabel", "{\\it f} [Hz]", ...
     "ylabel", "Faza [\\deg]"); 
  
  linkaxes([h.ax1, h.ax2], "x");
  guidata (obj, h);
  
  
endfunction